// Only modify this file to include
// - function definitions (prototypes)
// - include files
// - extern variable definitions
// In the appropriate section

#ifndef mwii21_H_
#define mwii21_H_
#include "Arduino.h"
#include "config.h"
#include "def.h"

//add your includes for the project mwii21 here

//end of add your includes here
#ifdef __cplusplus
extern "C" {
#endif
void loop();
void setup();
#ifdef __cplusplus
} // extern "C"
#endif


//add your function definitions for the project mwii21 here
#define  VERSION  210

/*********** RC alias *****************/
#define ROLL       0
#define PITCH      1
#define YAW        2
#define THROTTLE   3
#define AUX1       4
#define AUX2       5
#define AUX3       6
#define AUX4       7

#define PIDALT     3
#define PIDPOS     4
#define PIDPOSR    5
#define PIDNAVR    6
#define PIDLEVEL   7
#define PIDMAG     8
#define PIDVEL     9 // not used currently

#define BOXACC       0
#define BOXBARO      1
#define BOXMAG       2
#define BOXCAMSTAB   3
#define BOXCAMTRIG   4
#define BOXARM       5
#define BOXGPSHOME   6
#define BOXGPSHOLD   7
#define BOXPASSTHRU  8
#define BOXHEADFREE  9
#define BOXBEEPERON  10
#define BOXLEDMAX    11 // we want maximum illumination
#define BOXLLIGHTS   12 // enable landing lights at any altitude
#define BOXHEADADJ   13 // acquire heading for HEADFREE mode

// ******************
// rc functions
// ******************
#define MINCHECK 1100
#define MAXCHECK 1900

  // default POSHOLD control gains
  #define POSHOLD_P              .11
  #define POSHOLD_I              0.0
  #define POSHOLD_IMAX           20        // degrees

  #define POSHOLD_RATE_P         2.0
  #define POSHOLD_RATE_I         0.08      // Wind control
  #define POSHOLD_RATE_D         0.045     // try 2 or 3 for POSHOLD_RATE 1
  #define POSHOLD_RATE_IMAX      20        // degrees

  // default Navigation PID gains
  #define NAV_P                  1.4
  #define NAV_I                  0.20      // Wind control
  #define NAV_D                  0.08      //
  #define NAV_IMAX               20        // degrees

extern const char boxnames[];
extern const char pidnames[];

extern int16_t  debug[4];

extern uint16_t cycleTime;     // this is the number in micro second to achieve a full loop, it can differ a little and is taken into account in the PID loop
extern int16_t  headFreeModeHold;
extern uint8_t  vbat;               // battery voltage in 0.1V steps
extern uint16_t intPowerMeterSum, intPowerTrigger1;


extern int16_t  i2c_errors_count;
extern int16_t  annex650_overrun_count;
extern uint16_t calibratingG;

// **************
// gyro+acc IMU
// **************
extern int16_t gyroData[3];
extern int16_t gyroZero[3];
extern int16_t angle[2];  // absolute angle inclination in multiple of 0.1 degree    180 deg = 1800

extern int16_t rcData[8];          // interval [1000;2000]
extern int16_t rcCommand[4];       // interval [1000;2000] for THROTTLE and [-500;+500] for ROLL/PITCH/YAW
extern int16_t lookupPitchRollRC[6];// lookup table for expo & RC rate PITCH+ROLL
extern int16_t lookupThrottleRC[11];// lookup table for expo & mid THROTTLE


// *************************
// motor and servo functions
// *************************
extern int16_t axisPID[3];
extern int16_t motor[]; //int16_t motor[NUMBER_MOTOR];
#if defined(SERVO)
extern int16_t servo[8];
#endif

#define PIDITEMS 10
#define CHECKBOXITEMS 14

// ************************
// EEPROM Layout definition
// ************************
extern uint8_t dynP8[3], dynD8[3];
struct conf_t {
  uint8_t checkNewConf;
  uint8_t P8[PIDITEMS], I8[PIDITEMS], D8[PIDITEMS];
  uint8_t rcRate8;
  uint8_t rcExpo8;
  uint8_t rollPitchRate;
  uint8_t yawRate;
  uint8_t dynThrPID;
  uint8_t thrMid8;
  uint8_t thrExpo8;
  int16_t accZero[3];
  int16_t magZero[3];
  int16_t angleTrim[2];
  uint16_t activate[CHECKBOXITEMS];
  uint8_t powerTrigger1;
  #ifdef FLYING_WING
    uint16_t wing_left_mid;
    uint16_t wing_right_mid;
  #endif
  #ifdef TRI
    uint16_t tri_yaw_middle;
  #endif
  #if defined HELICOPTER || defined(AIRPLANE)|| defined(SINGLECOPTER)|| defined(DUALCOPTER)
    int16_t servoTrim[8];
  #endif
  #if defined(GYRO_SMOOTHING)
    uint8_t Smoothing[3];
  #endif
};
extern conf_t conf;

struct flags_struct {
  uint8_t OK_TO_ARM :1 ;
  uint8_t ARMED :1 ;
  uint8_t I2C_INIT_DONE :1 ; // For i2c gps we have to now when i2c init is done, so we can update parameters to the i2cgps from eeprom (at startup it is done in setup())
  uint8_t ACC_CALIBRATED :1 ;
  uint8_t NUNCHUKDATA :1 ;
  uint8_t ACC_MODE :1 ;
  uint8_t MAG_MODE :1 ;
  uint8_t BARO_MODE :1 ;
  uint8_t GPS_HOME_MODE :1 ;
  uint8_t GPS_HOLD_MODE :1 ;
  uint8_t HEADFREE_MODE :1 ;
  uint8_t PASSTHRU_MODE :1 ;
  uint8_t GPS_FIX :1 ;
  uint8_t GPS_FIX_HOME :1 ;
  uint8_t SMALL_ANGLES_25 :1 ;
  uint8_t CALIBRATE_MAG :1 ;
};

extern flags_struct f;

extern int16_t  gyroADC[3];
extern int16_t  accADC[3];
extern int16_t  accSmooth[3];
extern int16_t  magADC[3];

extern uint8_t  rcOptions[CHECKBOXITEMS];

extern uint16_t calibratingA;  // the calibration is done in the main loop. Calibrating decreases at each cycle down to 0, then we enter in a normal mode.
extern uint16_t acc_1G;             // this is the 1G measured acceleration

extern uint32_t currentTime;
extern uint16_t previousTime;

extern int32_t  BaroAlt;
extern int16_t  acc_25deg;

extern int16_t  heading,magHold;
extern int16_t  BaroPID;
extern int32_t  AltHold;
extern int16_t  errorAltitudeI;
extern int32_t  EstAlt;             // in cm

//#if defined(BUZZER)
  extern uint8_t  toggleBeep;
  extern uint8_t  buzzerFreq;         // delay between buzzer ring
//#endif


#define LAT  0
#define LON  1


extern int32_t  GPS_home[2];
extern int32_t  GPS_hold[2];
extern uint8_t  GPS_numSat;
extern uint16_t GPS_altitude,GPS_speed;                      // altitude in 0.1m and speed in 0.1m/s
extern uint8_t  GPS_update;                              // it's a binary toogle to distinct a GPS position update
extern int32_t  GPS_coord[2];
extern int16_t  GPS_angle[2];                      // it's the angles that must be applied for GPS correction
// The desired bank towards North (Positive) or South (Negative) : latitude
// The desired bank towards East (Positive) or West (Negative)   : longitude
extern int16_t  nav[2];
extern int16_t  nav_rated[2];    //Adding a rate controller to the navigation to make it smoother
extern uint16_t GPS_distanceToHome;                          // distance to home in meters
extern int16_t  GPS_directionToHome;                         // direction to home in degrees
extern uint16_t GPS_ground_course;                       // degrees*10
extern uint8_t  GPS_Enable;

#if defined(POWERMETER)
#define PMOTOR_SUM 8                     // index into pMeter[] for sum
extern uint32_t pMeter[PMOTOR_SUM + 1];  // we use [0:7] for eight motors,one extra for sum
extern uint8_t pMeterV;                  // dummy to satisfy the paramStruct logic in ConfigurationLoop()
extern uint32_t pAlarm;                  // we scale the eeprom value from [0:255] to this value we can directly compare to the sum in pMeter[6]
extern uint16_t powerValue;          // last known current
#endif

//for log
#if defined(LOG_VALUES) || defined(LCD_TELEMETRY)
extern uint16_t cycleTimeMax;       // highest ever cycle timen
extern uint16_t cycleTimeMin;   // lowest ever cycle timen
extern uint16_t powerMax;           // highest ever current
extern uint32_t armedTime;
extern int32_t  BAROaltStart;       // offset value from powerup
extern int32_t	BAROaltMax;	        // maximum value
#endif

#ifdef FAILSAFE
extern int16_t failsafeEvents;
#endif

#if defined(LCD_TELEMETRY)
extern uint8_t telemetry;
extern uint8_t telemetry_auto;
#endif


void blinkLED(uint8_t num, uint8_t wait,uint8_t repeat);
void annexCode();

extern volatile int16_t failsafeCnt;

//Do not add code below this line
#endif /* mwii21_H_ */
